@@ -475,7 +475,7 @@ trait ResponseExt: Sized {
     async fn error_for_status2(self) -> anyhow::Result<bytes::Bytes> {
         let (status, bytes) = self.split_response().await?;
 
-        log::trace!("{}", String::from_utf8_lossy(&bytes));
+        log::trace!("{}", String::from_utf8_lossy(&bytes).trim());
 
         if status.is_server_error() {
             anyhow::bail!("Server error {}", status)
