@@ -11,62 +11,30 @@ use tokio::process::Command as AsyncCmd;
 use crate::traits::Show;
 use crate::v0;
 
-#[derive(Debug)]
+#[derive(Clone, Copy, Debug)]
 pub(crate) enum Helm {
-    Skip {
-        namespace: String,
-        default_state: Option<String>,
-    },
-    Do {
-        namespace: String,
-        default_state: Option<String>,
-    },
+    Skip,
+    Do,
 }
 
 impl Helm {
-    pub(crate) fn new(namespace: String, default_state: Option<String>, skip_helm: bool) -> Self {
+    pub(crate) fn new(skip_helm: bool) -> Self {
         if skip_helm {
-            Self::Skip {
-                namespace,
-                default_state,
-            }
+            Self::Skip
+        } else if helm_is_found() {
+            Self::Do
         } else {
-            Self::Do {
-                namespace,
-                default_state,
-            }
+            log::warn!("helm is not detected. Will show helm command instead of executing them");
+            Self::Skip
         }
     }
 
-    pub(crate) fn skip(self) -> Self {
-        match self {
-            Self::Do {
-                namespace,
-                default_state,
-            } => Self::Skip {
-                namespace,
-                default_state,
-            },
-            other => other,
-        }
-    }
-
-    pub(crate) fn default_state(&self) -> Option<&str> {
-        match self {
-            Helm::Skip { default_state, .. } => default_state.as_deref(),
-            Helm::Do { default_state, .. } => default_state.as_deref(),
-        }
-    }
-
-    pub(crate) fn namespace(&self) -> &str {
-        match self {
-            Helm::Skip { namespace, .. } => namespace,
-            Helm::Do { namespace, .. } => namespace,
-        }
-    }
-
-    pub(crate) async fn execute(&self, cluster: &v0::Cluster) -> io::Result<(String, String)> {
-        let commands = self.command(cluster);
+    pub(crate) async fn execute(
+        &self,
+        cluster: &v0::Cluster,
+        namespace: &str,
+    ) -> io::Result<(String, String)> {
+        let commands = self.command(cluster, namespace);
         let (stdout, stderr) = match self {
             Helm::Skip { .. } => (
                 String::new(),
@@ -91,7 +59,7 @@ impl Helm {
         Ok((stdout, stderr))
     }
 
-    pub(crate) fn command(&self, cluster: &v0::Cluster) -> Vec<Command> {
+    pub(crate) fn command(&self, cluster: &v0::Cluster, namespace: &str) -> Vec<Command> {
         cluster
             .helm
             .iter()
@@ -100,7 +68,7 @@ impl Helm {
                 cmd.arg("install")
                     .arg(&helm.chart)
                     .arg("--namespace")
-                    .arg(self.namespace())
+                    .arg(namespace)
                     .arg("--repo")
                     .arg(&helm.repo)
                     .arg("--version")
@@ -121,3 +89,7 @@ impl Helm {
         Ok((output.status, stdout, stderr))
     }
 }
+
+fn helm_is_found() -> bool {
+    which::which("helm").is_ok()
+}
