@@ -24,7 +24,7 @@ use crate::Location;
 use crate::Output;
 
 use config::Config;
-use helper::AddLocation;
+use helper::{AddLocation, RegisterCluster};
 use print::{ClusterAndStates, Print, StateAndClusters};
 
 mod config;
@@ -101,7 +101,7 @@ enum Command {
     #[structopt(about = "Register new cluster", aliases = &["register-cl", "rc"], display_order(10))]
     RegisterCluster {
         #[structopt(help = "Cluster name, defaults to current k8s context")]
-        name: Option<v0::ClusterName>,
+        names: Vec<v0::ClusterName>,
 
         #[structopt(
             help = "List of states to for this cluster to use",
@@ -414,7 +414,7 @@ impl Cli {
             Command::ListStates => statehub.list_states().await,
             Command::ShowState { name } => statehub.show_state(&name).await,
             Command::RegisterCluster {
-                name,
+                names,
                 states,
                 no_state,
                 no_default_storage_class,
@@ -425,16 +425,19 @@ impl Cli {
                 provider,
                 cleanup_grace,
             } => {
-                let kubectl = if let Some(context) = name.as_deref() {
-                    k8s::Kubectl::with_context(context).await?
-                } else {
-                    k8s::Kubectl::new().await?
-                };
+                let kubectl = k8s::Kubectl::new().await?;
+                for name in &names {
+                    if !kubectl.has_context(name) {
+                        anyhow::bail!("Context {} is not found in kubeconfig", name)
+                    }
+                }
+
                 let no_default_storage_class = if no_state {
                     true
                 } else {
                     no_default_storage_class
                 };
+
                 let default_state = if no_default_storage_class {
                     None
                 } else if default_storage_class.is_none() {
@@ -442,19 +445,29 @@ impl Cli {
                 } else {
                     default_storage_class
                 };
+
                 let states = if no_state { None } else { Some(states) };
                 let claim_unowned_states = !no_state_owner;
-                let helm = k8s::Helm::new(namespace, default_state, skip_helm);
-                statehub
-                    .register_single_cluster(
-                        kubectl,
-                        provider,
-                        states,
-                        helm,
-                        claim_unowned_states,
-                        cleanup_grace,
-                    )
-                    .await
+
+                let helm = k8s::Helm::new(skip_helm);
+
+                let register_cluster = RegisterCluster::new(
+                    helm,
+                    namespace,
+                    states,
+                    default_state,
+                    claim_unowned_states,
+                    cleanup_grace,
+                );
+                if names.is_empty() {
+                    statehub
+                        .register_single_cluster(kubectl, provider, register_cluster)
+                        .await
+                } else {
+                    statehub
+                        .register_multiple_clusters(names, kubectl, provider, register_cluster)
+                        .await
+                }
             }
             Command::UnregisterCluster { force, name } => {
                 statehub.unregister_cluster(name, force).await
@@ -645,67 +658,61 @@ impl StateHub {
             .print(&self.stdout, self.json)
     }
 
-    async fn _register_multiple_clusters(&self) -> anyhow::Result<()> {
-        Ok(())
-    }
-
     async fn register_single_cluster(
         &self,
         kubectl: k8s::Kubectl,
         provider: Option<v0::Provider>,
-        states: Option<Vec<v0::StateName>>,
-        helm: k8s::Helm,
-        claim_unowned_states: bool,
-        cleanup_grace: String,
+        register_cluster: RegisterCluster,
     ) -> anyhow::Result<()> {
-        let helm = if k8s::helm_is_found() {
-            helm
-        } else {
-            log::warn!("helm is not detected. Will show helm command instead of executing them");
-            helm.skip()
-        };
-
         let locations = kubectl.collect_node_locations().await?;
 
-        if let Some(ref states) = states {
+        if let Some(states) = register_cluster.states() {
             self.adjust_all_states(states, &locations, false).await?;
         } else {
             self.verbosely("Skip adding this cluster to any state")?;
         }
 
-        let provider = if let Some(provider) = provider {
-            provider
-        } else {
-            kubectl.get_cluster_provider().await?
-        };
-
-        let cluster_name = kubectl.cluster_name();
-        let cluster = self
-            .api
-            .register_cluster(&cluster_name, provider, &locations)
+        self.register_cluster_helper(kubectl, provider, &locations, register_cluster)
             .await
-            .map(Quiet)?;
+    }
 
-        self.inform(format_args!(
-            "Registering {:#} cluster {} in {}",
-            provider,
-            cluster.name,
-            locations.show(),
-        ))?;
+    #[allow(clippy::too_many_arguments)]
+    async fn register_multiple_clusters(
+        &self,
+        names: Vec<v0::ClusterName>,
+        kubectl: k8s::Kubectl,
+        provider: Option<v0::Provider>,
+        register_cluster: RegisterCluster,
+    ) -> anyhow::Result<()> {
+        let mut clusters = indexmap::indexmap!();
 
-        k8s::validate_namespace(helm.namespace()).await?;
-
-        self.setup_cluster_token_helper(&cluster, &helm).await?;
-        self.setup_configmap_helper(&kubectl, &cluster, &helm, &cleanup_grace)
-            .await?;
-
-        if claim_unowned_states {
-            self.claim_unowned_states_helper(&cluster, states).await?;
+        for context in &names {
+            let kubectl = kubectl.clone_for_context(context);
+            let cluster_locations = kubectl.collect_node_locations().await?;
+            // Ideally I would store it as (kubectl => locations),
+            // but Kubeconfig doesn't impl Hash, so I cannot derive it on Kubectl
+            clusters.insert(cluster_locations, kubectl);
         }
 
-        self.helm_install_helper(&helm, &cluster).await?;
+        let locations = clusters
+            .keys()
+            .flatten()
+            .unique()
+            .copied()
+            .collect::<Vec<_>>();
 
-        cluster.print(&self.stdout, self.json)
+        if let Some(states) = register_cluster.states() {
+            self.adjust_all_states(states, &locations, false).await?;
+        } else {
+            self.verbosely("Skip adding this cluster to any state")?;
+        }
+
+        for (locations, kubectl) in clusters {
+            self.register_cluster_helper(kubectl, provider, &locations, register_cluster.clone())
+                .await?;
+        }
+
+        Ok(())
     }
 
     async fn show_cluster(&self, name: v0::ClusterName) -> anyhow::Result<()> {
@@ -883,7 +890,7 @@ impl StateHub {
     }
 
     async fn create_namespace(&self, namespace: String) -> anyhow::Result<()> {
-        k8s::validate_namespace(namespace)
+        k8s::validate_namespace(&namespace)
             .await
             .map(|namespace| println!("{:#?}", namespace))
     }
