@@ -4,6 +4,7 @@
 //
 
 use std::cmp;
+use std::iter;
 
 use chrono_humanize::HumanTime;
 use console::Emoji;
@@ -233,18 +234,19 @@ impl From<AzureRegion> for CreateStateLocationAzureDto {
     }
 }
 
-impl From<Vec<Location>> for CreateStateLocationsDto {
-    fn from(locations: Vec<Location>) -> Self {
-        let mut aws = vec![];
-        let mut azure = vec![];
-        for location in locations {
+impl iter::FromIterator<Location> for CreateStateLocationsDto {
+    fn from_iter<T>(iter: T) -> Self
+    where
+        T: IntoIterator<Item = Location>,
+    {
+        iter.into_iter().fold(Self::default(), |mut dto, location| {
             match location {
-                Location::Aws(region) => aws.push(region.into()),
-                Location::Azure(region) => azure.push(region.into()),
-                // Location::Gcp(region) => gcp.push(region),
+                Location::Aws(region) => dto.aws.push(region.into()),
+                Location::Azure(region) => dto.azure.push(region.into()),
+                // Location::Gcp(region) => dto.gcp.push(region),
             }
-        }
-        Self { aws, azure }
+            dto
+        })
     }
 }
 
